// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "DawnHostTux32.hpp"

#if DAWN_DEBUG_BUILD
  uint64_t dawnAllocatedItemCount;
#endif

using namespace Dawn;

int32_t main(int32_t argc, char **args) {
  int32_t result;

  #if DAWN_DEBUG_BUILD
    dawnAllocatedItemCount = 0;
  #endif

  // Create the host
  auto host = new DawnHost();
  auto game = new DawnGame(host);

  // Initialize the host and error check
  result = host->init(game);
  switch(result) {
    case DAWN_HOST_INIT_RESULT_SUCCESS:
      break;
    default:
      return result;
  }

  // Request the main loop to start running.
  result = host->start(game);
  switch(result) {
    case DAWN_HOST_START_RESULT_SUCCESS:
      break;
    case DAWN_HOST_START_RESULT_EXIT_SUCCESS:
      break;
    default:
      return result;
  }

  // Main loop finished without errors, cleanup
  host->unload(game);

  delete game;
  delete host;

  #if DAWN_DEBUG_BUILD
    assertTrue(dawnAllocatedItemCount == 0);
  #endif

  // Success
  return 0;
}